/* filename: loaddata.g
** description: reads time series data from ASCII data set into
** Gauss variable SERIES
** created:  05/03/00
**
** INPUTS: lmodel, lpath, mprior
*/

/* Time series, extracted 05/04/00
** 1. prodbls_log   : labor productivity, bls-hours, gdpq
** 2. prodblsic_log : labor productivity, bls-hours, c+i
** 3. prodlpm_log   : labor productivity, LPHMU, gdpq
** 4. prodlpmic_log : labor productivity, LPMHU, c+i
** 5. blshours_clog : bls hours
** 6. lpmhu_clog    : lphmu hours
** 7. gcdq_clog     : home-investment
** Quarterly Data from 1949:IV to 1997:IV
*/

load path = ^datapath data_q[193,7]= "homedat3.prn";

/* Compute growth rates for productivity and 
** consumption of durables: from 1950:I to 1997:IV
**
** sel = 1: bls, gdpq
**       2: bls, c+i
**       3: lpmhu, gdpq
**       4: lpmhu, c+i
*/
sel = 1;

series        = zeros(192,3);
seriesvec     = zeros(192,2);

ti            = seqa(1950,0.25,192);

if sel == 1;
   series[.,1]   = data_q[2:193,1] - data_q[1:192,1]; 
   series[.,2]   = data_q[2:193,7] - data_q[1:192,7]; 
   series[.,3]   = data_q[2:193,5];
   seriesvec[.,1]= data_q[1:192,1]; 
   seriesvec[.,2]= data_q[1:192,7];
elseif sel == 2;
   series[.,1]   = data_q[2:193,2] - data_q[1:192,2];
   series[.,2]   = data_q[2:193,7] - data_q[1:192,7]; 
   series[.,3]   = data_q[2:193,5];
   seriesvec[.,1]= data_q[1:192,2];
   seriesvec[.,2]= data_q[1:192,7];
elseif sel == 3;
   series[.,1]   = data_q[2:193,3] - data_q[1:192,3];
   series[.,2]   = data_q[2:193,7] - data_q[1:192,7]; 
   series[.,3]   = data_q[2:193,6];
   seriesvec[.,1]= data_q[1:192,3];
   seriesvec[.,2]= data_q[1:192,7];
else;
   series[.,1]   = data_q[2:193,4] - data_q[1:192,4];
   series[.,2]   = data_q[2:193,7] - data_q[1:192,7]; 
   series[.,3]   = data_q[2:193,6];
   seriesvec[.,1]= data_q[1:192,4];
   seriesvec[.,2]= data_q[1:192,7];
endif;

/* 
** 9/2001 version of paper:
** Delete first 20 observations (use 21:192)
** New Range is: 1955:I to 1997:IV
** 172 Observations
**
** 12/2002 version of paper:
** Delete first 56 observations (use 57:192)
** New Range is: 1964:I to 1997:IV
** 132 Observations
*/
series    = series[57:192,.];
seriesvec = seriesvec[57:192,.];
ti        = ti[57:192,.];

clear data_q, sel;
