/* filename:    homepmom.g
** description: Compute posterior mean and std and CI based on
**              output of posterior simulator
*/

new;
closeall;
library user, pgraph;
cls;
outwidth 128;

lmodel = "m1";
lprior = "1";
mhrun  = "1";

nburn       = 10001;        /* Number of initial draws to be discarded */
hpdprob     = 0.90;
nblock      = 10000;

/******************************************************************
**         Load Parameter Draws from MH Output
*/
#include c:\projects\active\homep\gauss\pathspec.g;


lpath  = parapath $+ "\\mhrun" $+ mhrun;
opath  = parapath $+ "\\mhrun" $+ mhrun;

ldraw  = lpath $+ "\\" $+ lmodel $+ lprior $+ "par";
odraws = opath $+ "\\" $+ lmodel $+ lprior $+ "pars";

open fhdraw = ^ldraw for read;

drawrow = readr( fhdraw, nburn); 
drawdim = cols(drawrow);
drawrow = seekr( fhdraw, nburn);

/* Part 1: Compute the mean of x(i) and x^2(i)
*/
drawmean   = zeros(1,drawdim);
drawsqmean = zeros(1,drawdim);
drawcross  = zeros(drawdim,drawdim);

eofloop = 0;
ndraws  = 0;

do until eofloop; 

   drawblock = readr( fhdraw, nblock );

   drawmean  = drawmean    + sumc(drawblock)';
   drawsqmean = drawsqmean + sumc(drawblock^2)';
   drawcross  = drawcross  + drawblock'*drawblock;  

   ndraws = ndraws + rows(drawblock);

   locate 1,1;
   "Part 1";
   "Draws " ndraws;

   eofloop = eof(fhdraw);

endo;

drawmean   = drawmean/ndraws;
drawsqmean = drawsqmean/ndraws;
drawstdd   = sqrt(drawsqmean - (drawmean)^2);
drawcov    = drawcross/ndraws - drawmean'*drawmean;

/* Report Posterior Mean and stderror
**
*/
"MODEL       " lmodel;
"PRIOR       " lprior;
"MH-Run      " mhrun;

"=============================================";
" " ;

load path = ^priopath para_names;
"Parameter | Mean  |  StdD |";
outmat = para_names'~ drawmean' ~ drawstdd' ;
let mask[1,3] = 0 1 1;
let fmt[3,3] =
   "-*.*s " 8 3
   "*.*lf " 8 3
   "*.*lf " 8 3;

d = printfm(outmat,mask,fmt);


/* Part 2: HPD Interval
*/
cls;
drawrow     = seekr( fhdraw, nburn); 
drawci      = zeros(2,drawdim);

j = 1;
do until j > drawdim;

   /* Read only the j'th column
   */
   drawrow   = seekr( fhdraw, nburn); 
   drawblock = readr( fhdraw, 1);

   drawcol = drawblock[1,j];
   eofloop = 0;
   ndraws  = 1;
   
   do until eofloop; 

      drawblock = readr( fhdraw, nblock );
      drawcol   = drawcol | drawblock[.,j];
      ndraws    = ndraws + rows(drawblock);
      eofloop   = eof(fhdraw);
      locate 1,1;
      "Part 2";
      "ndraws" ndraws;
      "Column" j;
         
   endo;
   
   drawci[.,j] = hpdint(drawcol,hpdprob);
   j = j+1;

endo;
clear drawcol;

/* Report Posterior Mean and stderror
**
*/
cls;
"MODEL       " lmodel;
"PRIOR       " lprior;
"MH-Run      " mhrun;
"Coverage    " hpdprob;  
"=============================================";
" " ;

load path = ^priopath para_names;
"Parameter | Mean  |  StdD | CI(LOW) | CI(HIGH)";
outmat = para_names'~ drawmean' ~ drawstdd' ~ drawci' ;
let mask[1,5] = 0 1 1 1 1;
let fmt[5,3] =
   "-*.*s " 8 3
   "*.*lf " 8 3
   "*.*lf " 8 3
   "*.*lf " 8 3
   "*.*lf " 8 3;

d = printfm(outmat,mask,fmt);

create fhdraws = ^odraws with DRAWSUM, drawdim, 8;
wr = writer(fhdraws, real(drawmean | drawstdd | drawcov | drawci) );
closeall fhdraws; 

end;

/**************************************************************/
/*                         Procedures                         */
/**************************************************************/

proc(1) = hpdint(draws,percent);
local     ndraws, drawdim, hpdband, nwidth, i, drawcoli,
          bup, minwidth, newwidth, j;

drawdim   = cols(draws);
ndraws    = rows(draws);
hpdband   = zeros(2,drawdim);
nwidth    = int(percent*ndraws);

i = 1;
do until i > drawdim;
   drawcoli = draws[.,i];
   /* sort response for period i, element 1 is max
   */
   drawcoli = rev(sortc(drawcoli,1));
   bup   = 1;
   minwidth  = drawcoli[1] - drawcoli[nwidth];
   j = 2;
   do until j > (ndraws-nwidth+1);
      newwidth = drawcoli[j] - drawcoli[j+nwidth-1];
      if newwidth < minwidth;
         bup = j;
         minwidth = newwidth;
      endif;
      j = j+1;
   endo;
   hpdband[2,i] = drawcoli[bup];
   hpdband[1,i] = drawcoli[bup+nwidth-1];
   i = i+1;
endo;
retp(hpdband);
endp;
