/* filename:    homesim.g
** description: The program simulates data from the home production model 
**              and writes them to the ASCII file simdata.prn
** created:     09/18/01
*/

new;
library pgraph, homelib, user;
cls;

/* Load information for the prior density
** We use prior mean parameter values to generate data
*/
#include c:\projects\active\homep\Gauss\pathspec.g
priorfile = "m11" $+ "pri.out";
npara     = 18;
mspec     = 2;

load path = ^priopath prior[npara,3] = ^priorfile;
para      = prior[1:npara,1];

sigb = para[npara];
siga = para[npara-1];
sigz = para[npara-2];

/* Specify the number of simulation draws
** used      = nsim
** discarded = ndrop
*/
nsim  = 100001;
ndrop = 500;

/* Call procedure to solve the model
*/
{ysim, valid} = dsgesim( para,mspec,nsim+ndrop,(sigz|siga|sigb) );
ysim = ysim[ndrop+1:ndrop+nsim,.];

outwidth 256;
outfile = datapath $+ "\\simdata.prn";
screen off;
output file = ^outfile reset;
ysim;
output file = ^outfile off;
screen on;
outwidth 80;

/******************************************/
/*          P R O C E D U R E S           */
/******************************************/

proc (2) = dsgesim( para,mspec,nsim,sig );
/* Procedure simulates sample path of DSGE model
** Returns:
**     model variables
**     valid
*/
local nvar,
      valid, retcode, TTT, RRR, ss, t, eps, gam,
      zt, ysim, produc, Ih, hours, ih_y, sst;

gam = para[3];
valid = 1;

/* Call procedure to solve the model
*/

{sst, retcode} = homesst(para, mspec);
{TTT, RRR, retcode} = homesolv(para,mspec);
ih_y = sst[6];

nvar = rows(TTT);
/* TTT is 17 by 17
** RRR is 17 by 3
*/

/* Simulate sample path
*/
ss = zeros(nsim+1,nvar);
zt = zeros(nsim+1,1);

t = 2;
do until t > nsim;

/* eps is made of epsz, epsa, epsb in this order;
** The path for zt is needed because of the initial rescaling
** We need to rescale eps by sig because eps is from a N(0,1)
*/

   eps = rndn(3,1);

   ss[t,.] = ( TTT*ss[t-1,.]' + RRR*( sig.*eps ) )';
   zt[t,.] = gam + zt[t-1,.] + eps[1]*sig[1];

   t = t+1;
endo;


/* Create output vector
** First we need the steady state since we solved with variables as
** deviation from the SS
** [dy(t), dy(t-1), dlm(t), dkm(t+1), dcm(t), da(t), db(t), dim(t),
**  dlh(t), dkh(t+1), dch(t), dih(t), dla(t), dxi(t), dpsi(t), dlm(t-1), dih(t-1)] : 17 variables
*/

produc = ss[.,1]  + zt - ss[.,3];
ih     = ln(ih_y) + ss[.,12] + zt;
hours  = 3.17     + ss[.,3];  
/* NOTE: the mean shift of hours corresponds to 
** average hours from 1955:I to 1959:IV
*/

ysim    = produc ~ ih ~ hours;

eosim:
retp(ysim, valid);
endp;
