/***************************************************************/
/*                                                             */
/*            Convert Impulse Response Functions               */
/*                                                             */
/*                       Frank Schorfheide                     */
/*                                                             */
/***************************************************************/

/* Filename:  irfconv.g
** created:   09/27/01
*/

new;
closeall;
library pgraph;
cls;

#include c:\projects\active\homep\gauss\pathspec.g;
nread   = 10000;

/* VEC - Posterior
*/
lprior = "1";
lmodel = "m0";
mhrun  = "1";
lpath = resupath $+ "\\mhrun" $+ mhrun $+ "\\";
opath = resupath $+ "\\mhrun" $+ mhrun $+ "\\";
gosub transform;


/* DSGE - Posterior
*/
lprior = "1";
lmodel = "m1";
mhrun  = "1";
lpath = resupath $+ "\\mhrun" $+ mhrun $+ "\\";
opath = resupath $+ "\\mhrun" $+ mhrun $+ "\\";
gosub transform;


/* VEC - Prior
*/
/*
lprior = "1";
lmodel = "m0";
mhrun  = "0";
lpath = resupath $+ "\\mhrun" $+ mhrun $+ "\\";
opath = resupath $+ "\\mhrun" $+ mhrun $+ "\\";
gosub transform;
*/

/* DSGE - Prior
*/
/*
lprior = "1";
lmodel = "m1";
mhrun  = "0";
lpath = resupath $+ "\\mhrun" $+ mhrun $+ "\\";
opath = resupath $+ "\\mhrun" $+ mhrun $+ "\\";
gosub transform;
*/
end;

/*****************************************************************
**  S U B R O U T I N E
*/
transform:

/* Open Input Files
*/
lir11 = lpath $+ lmodel $+ lprior $+ "r11" $+mhrun;
lir12 = lpath $+ lmodel $+ lprior $+ "r12" $+mhrun;
lir13 = lpath $+ lmodel $+ lprior $+ "r13" $+mhrun;

lir21 = lpath $+ lmodel $+ lprior $+ "r21" $+mhrun;
lir22 = lpath $+ lmodel $+ lprior $+ "r22" $+mhrun;
lir23 = lpath $+ lmodel $+ lprior $+ "r23" $+mhrun;

lir31 = lpath $+ lmodel $+ lprior $+ "r31" $+mhrun;
lir32 = lpath $+ lmodel $+ lprior $+ "r32" $+mhrun;
lir33 = lpath $+ lmodel $+ lprior $+ "r33" $+mhrun;

open fhir11 = ^lir11 for read;
open fhir12 = ^lir12 for read;
open fhir13 = ^lir13 for read;

open fhir21 = ^lir21 for read;
open fhir22 = ^lir22 for read;
open fhir23 = ^lir23 for read;

open fhir31 = ^lir31 for read;
open fhir32 = ^lir32 for read;
open fhir33 = ^lir33 for read;

irfrow = readr(fhir11,1);
nirf   = cols(irfrow);
irfrow = seekr(fhir11,1);

/* Initialize output files
*/
oic11 = opath $+ lmodel $+ lprior $+ "c11" $+ mhrun;
oic12 = opath $+ lmodel $+ lprior $+ "c12" $+ mhrun;
oic13 = opath $+ lmodel $+ lprior $+ "c13" $+ mhrun;

oic21 = opath $+ lmodel $+ lprior $+ "c21" $+ mhrun;
oic22 = opath $+ lmodel $+ lprior $+ "c22" $+ mhrun;
oic23 = opath $+ lmodel $+ lprior $+ "c23" $+ mhrun;

oic31 = opath $+ lmodel $+ lprior $+ "c31" $+ mhrun;
oic32 = opath $+ lmodel $+ lprior $+ "c32" $+ mhrun;
oic33 = opath $+ lmodel $+ lprior $+ "c33" $+ mhrun;

create fhic11 = ^oic11 with IR11, nirf, 8;
create fhic12 = ^oic12 with IR12, nirf, 8;
create fhic13 = ^oic13 with IR13, nirf, 8;

create fhic21 = ^oic21 with IR21, nirf, 8;
create fhic22 = ^oic22 with IR22, nirf, 8;
create fhic23 = ^oic23 with IR23, nirf, 8;

create fhic31 = ^oic31 with IR31, nirf, 8;
create fhic32 = ^oic32 with IR32, nirf, 8;
create fhic33 = ^oic33 with IR33, nirf, 8;

/* Convert Impulse Response Functions
*/

eofloop = 0;
ndraws  = 0;
loopct  = 0;

do until eofloop; 

   irf11sim = readr( fhir11, nread);
   irf12sim = readr( fhir12, nread);
   irf13sim = readr( fhir13, nread);

   irf21sim = readr( fhir21, nread);
   irf22sim = readr( fhir22, nread);
   irf23sim = readr( fhir23, nread);

   irf31sim = readr( fhir31, nread);
   irf32sim = readr( fhir32, nread);
   irf33sim = readr( fhir33, nread);

   mult1 = 2*((irf11sim[.,1] .GE 0) -0.5);
   mult2 = 2*((irf12sim[.,1] .GE 0) -0.5);
   mult3 = 2*((irf33sim[.,1] .GE 0) -0.5);



/* Normalization:
** response of prod to shock 1 > 0
**             prod to shock 2 > 0
**             hours to shock 3 < 0
*/


   irf11sim = irf11sim.*mult1;
   irf21sim = irf21sim.*mult1;
   irf31sim = irf31sim.*mult1;

   irf12sim = irf12sim.*mult2;
   irf22sim = irf22sim.*mult2;
   irf32sim = irf32sim.*mult2;

   irf13sim = -irf13sim.*mult3;
   irf23sim = -irf23sim.*mult3;
   irf33sim = -irf33sim.*mult3;


   wr = writer(fhic11,irf11sim);
   wr = writer(fhic12,irf12sim);
   wr = writer(fhic13,irf13sim);

   wr = writer(fhic21,irf21sim);
   wr = writer(fhic22,irf22sim);
   wr = writer(fhic23,irf23sim);

   wr = writer(fhic31,irf31sim);
   wr = writer(fhic32,irf32sim);
   wr = writer(fhic33,irf33sim);

   loopct = loopct + 1;
   locate 1,1;
   "Iteration" loopct;
/*
   irf11sim[.,1 3 5 10 25 40];
   irf12sim[.,1 3 5 10 25 40];
   irf13sim[.,1 3 5 10 25 40];
   k = keyw;
*/

   eofloop = eof(fhir11); 

endo;

closeall fhir11, fhir12, fhir13, 
         fhir21, fhir22, fhir23,
         fhir31, fhir32, fhir33;

return;
