/***************************************************************/
/*                                                             */
/*     VEC analysis: converting the parameter estimates        */
/*                                                             */
/*                       Frank Schorfheide                     */
/*                                                             */
/***************************************************************/

/* Filename:  vecpred.g
** created:   07/17/00
*/

/* new; */
closeall;
library pgraph;

/* Load specification information
*/
#include c:\projects\active\homep\Gauss\pathspec.g
#include c:\projects\active\homep\Gauss\vecspec.g

/* Define some constants
*/
nvar   = 3;
k      = 1+ttrend+1+p*nvar;
npara  = nvar*k + 0.5*nvar*(nvar+1) + 1;

nirf   = 40;                 
nma    = 10000;
nomega = 50;
omegagr= seqa(0.0625, 0.33*pi/(nomega-1), nomega);

nsim   = 100;

lmodel = "m0";
lprior = "1";
mhrun  = "1";
lpath  = parapath $+ "\\mhrun" $+ mhrun $+"\\";
opath  = resupath $+ "\\mhrun" $+ mhrun $+"\\";

/* Open files with VEC Parameter Draws
*/
lpara = lpath $+ lmodel  $+ lprior $+ "par";
open fhpara = ^lpara for read;

/* Initialize Output files
*/
oir11 = opath $+ lmodel $+ lprior $+ "r11" $+ mhrun;
oir12 = opath $+ lmodel $+ lprior $+ "r12" $+ mhrun;
oir13 = opath $+ lmodel $+ lprior $+ "r13" $+ mhrun;

oir21 = opath $+ lmodel $+ lprior $+ "r21" $+ mhrun;
oir22 = opath $+ lmodel $+ lprior $+ "r22" $+ mhrun;
oir23 = opath $+ lmodel $+ lprior $+ "r23" $+ mhrun;

oir31 = opath $+ lmodel $+ lprior $+ "r31" $+ mhrun;
oir32 = opath $+ lmodel $+ lprior $+ "r32" $+ mhrun;
oir33 = opath $+ lmodel $+ lprior $+ "r33" $+ mhrun;

ospy1 = opath $+ lmodel $+ lprior $+ "sy1" $+ mhrun;
ospy2 = opath $+ lmodel $+ lprior $+ "sy2" $+ mhrun;
ospy3 = opath $+ lmodel $+ lprior $+ "sy3" $+ mhrun;

ospl1 = opath $+ lmodel $+ lprior $+ "sl1" $+ mhrun;
ospl2 = opath $+ lmodel $+ lprior $+ "sl2" $+ mhrun;
ospl3 = opath $+ lmodel $+ lprior $+ "sl3" $+ mhrun;

ovard = opath  $+ lmodel $+ lprior $+ "vd"  $+ mhrun;

create fhir11 = ^oir11 with IR11, nirf, 8;
create fhir12 = ^oir12 with IR12, nirf, 8;
create fhir13 = ^oir13 with IR13, nirf, 8;

create fhir21 = ^oir21 with IR21, nirf, 8;
create fhir22 = ^oir22 with IR22, nirf, 8;
create fhir23 = ^oir23 with IR23, nirf, 8;

create fhir31 = ^oir31 with IR31, nirf, 8;
create fhir32 = ^oir32 with IR32, nirf, 8;
create fhir33 = ^oir33 with IR33, nirf, 8;

create fhspy1 = ^ospy1 with SPY1, nomega, 8;
create fhspy2 = ^ospy2 with SPY2, nomega, 8;
create fhspy3 = ^ospy3 with SPY3, nomega, 8;

create fhspl1 = ^ospl1 with SPL1, nomega, 8;
create fhspl2 = ^ospl2 with SPL2, nomega, 8;
create fhspl3 = ^ospl3 with SPL3, nomega, 8;

create fhvard = ^ovard with VARD, 6,    8;

cls;

/* Initialization
*/ 

eofloop = 0;
loopct  = 0;

do until eofloop; 

   /* Read blocks of size nsim
   ** and generate a vector of alphas
   ** of the same dimension
   */
   parasim = readr( fhpara, nsim );
   nread   = rows(parasim);

   if sh_alp == 1; /* BETA Prior */
      a_alp  = (1-m0_alp)*m0_alp^2/s0_alp^2 - m0_alp;
      b_alp  = a_alp*(1/m0_alp - 1);
      alp_sim = rndbeta(nread,1,a_alp,b_alp);
      
   elseif sh_alp == 2; /* GAMMA PRIOR */
      b_alp = s0_alp^2/m0_alp;
      a_alp = m0_alp/b_alp;
      alp_sim = b_alp*rndgam(nread,1,a_alp);
      
   elseif sh_alp == 3; /* GAUSSIAN PRIOR */
      a_alp = m0_alp;
      b_alp = s0_alp;
      alp_sim = a_alp + b_alp*rndn(nread,1);
      
   elseif sh_alp == 0; /* No Prior - Fixed */
      alp_sim = m0_alp;

   endif;
 
   Asim11 = zeros(nread,nirf); 
   Asim12 = zeros(nread,nirf);
   Asim13 = zeros(nread,nirf);

   Asim21 = zeros(nread,nirf);
   Asim22 = zeros(nread,nirf);
   Asim23 = zeros(nread,nirf);

   Asim31 = zeros(nread,nirf);
   Asim32 = zeros(nread,nirf);
   Asim33 = zeros(nread,nirf);

   Ssimy1 = zeros(nread,nomega);
   Ssimy2 = zeros(nread,nomega);
   Ssimy3 = zeros(nread,nomega);

   Ssiml1 = zeros(nread,nomega);
   Ssiml2 = zeros(nread,nomega);
   Ssiml3 = zeros(nread,nomega);

   eigvsim = zeros(nread,1);

   vardsim   = zeros(nread,6);

   j = 1;
   do until j > nread;

     la_sim  = parasim[j,1];
     sig_sim = zeros(3,3);
     sig_sim[1,1:3] = parasim[j,2:4];
     sig_sim[1:3,1] = parasim[j,2:4]';
     sig_sim[2,2:3] = parasim[j,5:6];
     sig_sim[2:3,2] = parasim[j,5:6]';
     sig_sim[3,3]   = parasim[j,7];
     
     C_sim   = parasim[j,8:npara]';
     CCt_sim = reshape(C_sim,nvar,k);
     CC_sim  = CCt_sim';

     alp_simj = alp_sim[j];

     /* Identification Procedure
     */

     /* Compute Long-run effect
     */
     alpha     = CCt_sim[.,1];
     alphaorth = ( -alpha[2] ~ - alpha[3] )|
                 ( alpha[1]  ~   0        )|
                 ( 0         ~   alpha[1] );
     beta      = 1 | -la_sim | 0;
     betaorth  = (la_sim ~ 0 )|
                 (1    ~ 0   )|
                 (0    ~ 1   );
           
     gam = eye(nvar);

     if ttrend == 0;
       i = 1;
       do until i > p;
         gam = gam - CCt_sim[.,3+(i-1)*nvar:2+i*nvar];
         i = i+1;
       endo;
     else;
       i = 1;
       do until i > p;
         gam = gam - CCt_sim[.,4+(i-1)*nvar:3+i*nvar];
         i = i+1;
       endo;
     endif;

     C1_sim = betaorth*inv( alphaorth'*gam*betaorth )*alphaorth';

     /* Calculate the coefficients of PHI0-matrix
     */
     psi0_sim = chol(sig_sim)';

     aux1     = C1_sim*psi0_sim;
     aux1     = aux1[1,.];
     aux2     = (1 ~ 0 ~ -(alp_simj-1))*psi0_sim;

     b3sim    = inv( aux1  | aux2  | ones(1,3) )*(0|0|1);
     b3sim    = b3sim/sqrt(b3sim'*b3sim);
     b2sim    = inv( aux1  | b3sim'| ones(1,3) )*(0|0|1);
     b2sim    = b2sim/sqrt(b2sim'*b2sim);
     b1sim    = inv( b2sim'| b3sim'| ones(1,3) )*(0|0|1);
     b1sim    = b1sim/sqrt(b1sim'*b1sim);

     bsim = b1sim~b2sim~b3sim;

     phi0_sim = psi0_sim * bsim;     
     phi0_sim = phi0_sim .* (1-2*(phi0_sim[1,.].<0) );

     /* Compute the impulse response functions
     */

     {dydz_irf, ydz_irf} = vecirf(la_sim,CCt_sim,Phi0_sim,(100|0|0),nirf);
     Asim11[j,.] = ydz_irf[1,1:nirf];
     Asim21[j,.] = ydz_irf[2,1:nirf];
     Asim31[j,.] = dydz_irf[3,1:nirf];

     {dyda_irf, yda_irf} = vecirf(la_sim,CCt_sim,Phi0_sim,(0|100|0),nirf);
     Asim12[j,.] = yda_irf[1,1:nirf];
     Asim22[j,.] = yda_irf[2,1:nirf];
     Asim32[j,.] = dyda_irf[3,1:nirf];

     {dydb_irf, ydb_irf} = vecirf(la_sim,CCt_sim,Phi0_sim,(0|0|100),nirf);
     Asim13[j,.] = ydb_irf[1,1:nirf];
     Asim23[j,.] = ydb_irf[2,1:nirf];
     Asim33[j,.] = dydb_irf[3,1:nirf];

     if ttrend == 0;
        CCt_sim = CCt_sim[.,1] ~ CCt_sim[.,3:cols(CCt_sim)];
     elseif ttrend == 1;
        CCt_sim = CCt_sim[.,1] ~ CCt_sim[.,4:cols(CCt_sim)];
     endif;

     vardl = zeros(1,3);
     vardy = zeros(1,3);

     {spec,gam,maxeig} = vecspec(la_sim, CCt_sim, Phi0_sim, (1|0|0) );
     Ssimy1[j,.] = spec[1,.];
     Ssiml1[j,.] = spec[2,.];
     vardl[1,1] = gam[2,2];
     vardy[1,1] = gam[1,1];

     {spec,gam,maxeig} = vecspec(la_sim, CCt_sim, Phi0_sim, (0|1|0) );
     Ssimy2[j,.] = spec[1,.];
     Ssiml2[j,.] = spec[2,.];
     vardl[1,2] = gam[2,2];
     vardy[1,2] = gam[1,1];

     {spec,gam,maxeig} = vecspec(la_sim, CCt_sim, Phi0_sim, (0|0|1) );
     Ssimy3[j,.] = spec[1,.];
     Ssiml3[j,.] = spec[2,.];
     vardl[1,3] = gam[2,2];
     vardy[1,3] = gam[1,1];

       
     vardl = vardl/sumc(vardl');
     vardy = vardy/sumc(vardy');
     vardsim[j,.] = vardy~vardl;

     /* Screen output
     */
     /*
     locate 1,1;
     "Simulation step" j "out of" nsim;

     "Cointvector    " la_sim;
     "Alpha          " alp_simj;
     "Maximum Eigv   " maxeig;
     "Variance D (Y) " vardy;
     "Variance D (L) " vardl;

     if maxeig ge 1;
        eigvsim[j,.] = 1;
     endif;
     */
     j = j+1;

  endo;

  /* Eliminate Draws from explosive models
  */
  Asim11= delif(Asim11,eigvsim.>0);
  Asim12= delif(Asim12,eigvsim.>0);
  Asim13= delif(Asim13,eigvsim.>0);

  Asim21= delif(Asim21,eigvsim.>0);
  Asim22= delif(Asim22,eigvsim.>0);
  Asim23= delif(Asim23,eigvsim.>0);

  Asim31= delif(Asim31,eigvsim.>0);
  Asim32= delif(Asim32,eigvsim.>0);
  Asim33= delif(Asim33,eigvsim.>0);

  Ssimy1= delif(Ssimy1,eigvsim.>0);
  Ssimy2= delif(Ssimy2,eigvsim.>0);
  Ssimy3= delif(Ssimy3,eigvsim.>0);

  Ssiml1= delif(Ssiml1,eigvsim.>0);
  Ssiml2= delif(Ssiml2,eigvsim.>0);
  Ssiml3= delif(Ssiml3,eigvsim.>0);

  vardsim= delif(vardsim,eigvsim.>0);
 
  /* Save the results
  */
  wr = writer(fhir11,Asim11);
  wr = writer(fhir12,Asim12);
  wr = writer(fhir13,Asim13);

  wr = writer(fhir21,Asim21);
  wr = writer(fhir22,Asim22);
  wr = writer(fhir23,Asim23);

  wr = writer(fhir31,Asim31);
  wr = writer(fhir32,Asim32);
  wr = writer(fhir33,Asim33);

  wr = writer(fhspy1,Ssimy1);
  wr = writer(fhspy2,Ssimy2);
  wr = writer(fhspy3,Ssimy3);

  wr = writer(fhspl1,Ssiml1);
  wr = writer(fhspl2,Ssiml2);
  wr = writer(fhspl3,Ssiml3);

  wr = writer(fhvard,vardsim);

  loopct = loopct + 1;
  locate 10,1;
  "Loop     " loopct;
  "Discarded" meanc(eigvsim);
  eofloop = eof(fhpara);

endo;

closeall fhir11, fhir12, fhir13,
         fhir21, fhir22, fhir23,
         fhir31, fhir32, fhir33,
         fhspy1, fhspy2, fhspy3,
         fhspl1, fhspl2, fhspl3,
         fhvard, fhpara;

clear Asim11, Asim12, Asim13,
      Asim21, Asim22, Asim23,
      Asim31, Asim32, Asim33,
      Ssimy1, Ssimy2, Ssimy3,
      Ssiml1, Ssiml2, Ssiml3,
      vardsim, corrsim;



/*********************************************************
**      P R O C E D U R E S
*/

proc (3) = vecspec(lambda,CCt,Phi0,sig);

local CCtt, sigma, Phi0t, 
      mu, mu_orth, la, la_orth, labar, laorthbar,
      phistar, phistar0, i, rr, nstate, Pt,
      MM1, MM2, ZZ, TT, S1, spec, gam, maxeigphi;

/* Expand the VAR model by Delta(L),
** assume that the constants are removed
** first column contains VEC coefficients
** fourth column contains coefficients on lagged L
*/

CCtt      = CCt | CCt[3,.]  ;
CCtt[4,4] = CCtt[4,4]-1     ;
sigma     = eye(3).*sig     ;

Phi0t     = Phi0 | Phi0[3,.];

/* construct mu_orth and la_orth
*/
mu =       CCtt[.,1];
mu_orth =  ( -mu[2] | mu[1] | 0 | 0 ) ~
           ( 0 | -mu[3] | mu[2] | 0 ) ~
           ( 0 | 0 | -mu[4] | mu[3] ) ;

la =       ( 1 | -lambda| 0 | 0 );
la_orth =  ( lambda | 1 | 0 | 0 ) ~
           ( 0      | 0 | 1 | 0 ) ~
           ( 0      | 0 | 0 | 1 );

labar     = la*inv(la'*la);
laorthbar = la_orth*inv(la_orth'*la_orth);


/* Set up companion form matrix
*/

phistar0 = labar ~ laorthbar;

i = 1;
phistar = inv(phistar0)*( (mu + labar + ( CCtt[.,2:4]~zeros(4,1) )* labar) ~ (CCtt[.,2:4]~zeros(4,1))*laorthbar);
if p > 1;
   do until i == p;
      phistar = phistar ~ inv(phistar0)* ( ( (CCtt[.,(1+nvar*i+1):(1+nvar*(i+1))]~zeros(4,1) )  
                                           - (CCtt[.,(1+nvar*(i-1)+1):(1+nvar*i)]~zeros(4,1) ) )*labar
                                       ~ (CCtt[.,(1+nvar*i+1):(1+nvar*(i+1))]~zeros(4,1))*laorthbar );
      i = i+1;
   endo;
endif;
phistar = phistar ~ inv(phistar0)* ( ( -CCtt[.,1+nvar*(i-1)+1:1+nvar*i] ~ zeros(4,1) )*labar ~ zeros(4,3) );

phi0t = inv(phistar0)*phi0t;

phistar = phistar | (eye(cols(phistar)-4) ~ zeros(cols(phistar)-4,4) );
rr      = phi0t   | zeros(cols(phistar)-4,3) ;
nstate  = cols(phistar);


Pt = reshape(inv(eye(nstate^2) - (Phistar.*.Phistar))*vec(RR*SIGMA*RR'),nstate,nstate);
maxeigphi = maxc(abs(eig(PHIstar)));

 
/* Define Selection Matrices
*/
MM1 = ( 1 ~ 0 ~ 0 ~ 1)|
      ( 0 ~ 0 ~ 1 ~ 0);

MM2 = zeros(4,nstate);
MM2[.,1:4] = labar ~ laorthbar;
MM2[.,5]   = -labar;

ZZ = MM1*MM2;
TT = phistar;

spec = zeros(2,nomega);

i = 1;
do until i > nomega;
   
   S1 = complex( eye(nstate) - TT*cos(omegagr[i]),   TT*sin(omegagr[i]) );  
   spec[.,i] = real( diag( ZZ* (inv(S1)*(RR*sigma*RR')*inv(S1)'/(2*pi) )* ZZ' ) );

   i = i+1;

endo;

Gam = ZZ*Pt*ZZ';

retp(spec,gam,maxeigphi);
endp;

/****************************************************************/

proc (2) = vecirf(lambda,CCt,Phi0,eps,nirf);
/* Compute impulse response functions
*/
local nvar, nlags, beta, CCts, dy_irf, y_irf, XXs, i, impact;

nvar  = rows(CCt);
nlags = (cols(CCt)-2-ttrend)/nvar;
beta  = (1|-lambda|0);
impact = 1 | zeros(nirf-1,1);

/* eliminate the constant term
*/
if ttrend == 0;
   CCts    = CCt[.,1 3:cols(CCt)];
   dy_irf  = zeros(nvar,nirf);
   y_irf   = zeros(nvar,nirf);
   XXs     = zeros(1+nvar*nlags,1);
else;
   CCts    = CCt[.,1 4:cols(CCt)];
   dy_irf  = zeros(nvar,nirf);
   y_irf   = zeros(nvar,nirf);
   XXs     = zeros(1+nvar*nlags,1);
endif;


i = 1;
do until i > nirf;

   dy_irf[.,i] = CCts*XXs + impact[i]*(Phi0*eps);
   if i == 1;
      y_irf[.,i] = dy_irf[.,i];
   else; 
      y_irf[.,i]  = y_irf[.,i-1]+dy_irf[.,i];
   endif;
   
   /* Update the X-vector
   */
   XXs[1] = beta'*y_irf[.,i];
   if nlags > 1;
      XXs[2+nvar:1+nlags*nvar] = XXs[2:1+(nlags-1)*nvar];
   endif;
   XXs[2:1+nvar] = dy_irf[.,i];
   i = i+1;

endo;

retp(dy_irf,y_irf);
endp;


