/***************************************************************/
/*                                                             */
/*     VEC analysis: converting the parameter estimates        */
/*                                                             */
/*                       Frank Schorfheide                     */
/*                                                             */
/***************************************************************/

/* Filename:  vecpred2.g
** created:   09/27/01
*/

new; 
closeall;
library pgraph;

/* Load specification information
*/
#include c:\projects\active\homep\Gauss\pathspec.g
#include c:\projects\active\homep\Gauss\vecspec.g

nsim   = 110;

nalp    = 3;
alpgrid = 0.3 | 0.6 | 0.9;

/* Define some constants
*/
nvar   = 3;
k      = 1+ttrend+1+p*nvar;
npara  = nvar*k + 0.5*nvar*(nvar+1) + 1;

nma    = 10000;
nomega = 50;
omegagr= seqa(0.0625, 0.33*pi/(nomega-1), nomega);
spectr = 2-2*cos(omegagr);
spectr = spectr';

lmodel = "m0";
lprior = "1";
mhrun  = "1";
lpath  = parapath $+ "\\mhrun" $+ mhrun $+"\\";
opath  = resupath $+ "\\mhrun" $+ mhrun $+"\\";

/* Open files with VEC Parameter Draws
*/
lpara = lpath $+ "\\" $+ lmodel  $+ lprior $+ "par";
open fhpara = ^lpara for read;

/* Initialize Output files
*/
ovdya  =  opath $+ lmodel $+ lprior $+ "vdya";
ovdla  =  opath $+ lmodel $+ lprior $+ "vdla";

create fhvdya = ^ovdya with VDY, 2*nalp, 8;
create fhvdla = ^ovdla with VDL, 2*nalp, 8;

cls;

/* Initialization
*/ 

eofloop   = 0;
loopct    = 0;

do until eofloop; 

   /* Read blocks of size nsim
   ** and generate a vector of alphas
   ** of the same dimension
   */
   parasim = readr( fhpara, nsim );
   nread   = rows(parasim);

   VDYAsim  = zeros(nread,2*nalp); 
   VDLAsim  = zeros(nread,2*nalp); 
   VDinvalid = zeros(nread,1);
    
   j = 1;
   do until j > nread;

     la_sim  = parasim[j,1];
     sig_sim = zeros(3,3);
     sig_sim[1,1:3] = parasim[j,2:4];
     sig_sim[1:3,1] = parasim[j,2:4]';
     sig_sim[2,2:3] = parasim[j,5:6];
     sig_sim[2:3,2] = parasim[j,5:6]';
     sig_sim[3,3]   = parasim[j,7];
     
     alp_ind = 1;
     do until alp_ind > nalp;

        C_sim   = parasim[j,8:npara]';
        CCt_sim = reshape(C_sim,nvar,k);
        CC_sim  = CCt_sim';

        alp_simj = alpgrid[alp_ind];

        /* Identification Procedure
        */

        /* Compute Long-run effect
        */
        alpha     = CCt_sim[.,1];
        alphaorth = ( -alpha[2] ~ - alpha[3] )|
                    ( alpha[1]  ~   0        )|
                    ( 0         ~   alpha[1] );
        beta      = 1 | -la_sim | 0;
        betaorth  = (la_sim ~ 0 )|
                    (1    ~ 0   )|
                    (0    ~ 1   );
           
        gam = eye(nvar);

        if ttrend == 0;
          i = 1;
          do until i > p;
             gam = gam - CCt_sim[.,3+(i-1)*nvar:2+i*nvar];
             i = i+1;
          endo;
        else;
          i = 1;
          do until i > p;
             gam = gam - CCt_sim[.,4+(i-1)*nvar:3+i*nvar];
             i = i+1;
          endo;
        endif;

        C1_sim = betaorth*inv( alphaorth'*gam*betaorth )*alphaorth';

        /* Calculate the coefficients of PHI0-matrix
        */
        psi0_sim = chol(sig_sim)';

        aux1     = C1_sim*psi0_sim;
        aux1     = aux1[1,.];
        aux2     = (1 ~ 0 ~ -(alp_simj-1))*psi0_sim;

        b3sim    = inv( aux1  | aux2  | ones(1,3) )*(0|0|1);
        b3sim    = b3sim/sqrt(b3sim'*b3sim);
        b2sim    = inv( aux1  | b3sim'| ones(1,3) )*(0|0|1);
        b2sim    = b2sim/sqrt(b2sim'*b2sim);
        b1sim    = inv( b2sim'| b3sim'| ones(1,3) )*(0|0|1);
        b1sim    = b1sim/sqrt(b1sim'*b1sim);

        bsim = b1sim~b2sim~b3sim;

        phi0_sim = psi0_sim * bsim;     
        phi0_sim = phi0_sim .* (1-2*(phi0_sim[1,.].<0) );

        if ttrend == 0;
          CCt_sim = CCt_sim[.,1] ~ CCt_sim[.,3:cols(CCt_sim)];
        elseif ttrend == 1;
          CCt_sim = CCt_sim[.,1] ~ CCt_sim[.,4:cols(CCt_sim)];
        endif;

        vardl = zeros(1,3);
        vardy = zeros(1,3);
        Ssimy = zeros(3,1);
        Ssiml = zeros(3,1);

        {spec,gam,maxeig} = vecspec(la_sim, CCt_sim, Phi0_sim, (1|0|0) );
        Ssimy[1,.] = sumc((spec[1,.]./spectr)');
        Ssiml[1,.] = sumc((spec[2,.]./spectr)');
        vardl[1,1] = gam[2,2];
        vardy[1,1] = gam[1,1];

        {spec,gam,maxeig} = vecspec(la_sim, CCt_sim, Phi0_sim, (0|1|0) );
        Ssimy[2,.] = sumc((spec[1,.]./spectr)');
        Ssiml[2,.] = sumc((spec[2,.]./spectr)');
        vardl[1,2] = gam[2,2];
        vardy[1,2] = gam[1,1];

        {spec,gam,maxeig} = vecspec(la_sim, CCt_sim, Phi0_sim, (0|0|1) );
        Ssimy[3,.] = sumc((spec[1,.]./spectr)');
        Ssiml[3,.] = sumc((spec[2,.]./spectr)');
        vardl[1,3] = gam[2,2];
        vardy[1,3] = gam[1,1];

        vardl = vardl/sumc(vardl');
        vardy = vardy/sumc(vardy');
      
        Ssimy = Ssimy./sumc(Ssimy)';
        Ssiml = Ssiml./sumc(Ssiml)';
        
        if maxeig le 1;
           VDYAsim[j,alp_ind]      = Ssimy[3,.]; 
           VDYAsim[j,alp_ind+nalp] = vardy[1,3]; 
           VDLAsim[j,alp_ind]      = Ssiml[3,.]; 
           VDLAsim[j,alp_ind+nalp] = vardl[1,3]; 
        else;
           VDinvalid[j,1] = 1;
        endif;
        
        alp_ind = alp_ind+1;
        
     endo;
     j = j + 1;
     
  endo;

  VDYAsim = delif(VDYAsim, VDinvalid );
  VDLAsim = delif(VDLAsim, VDinvalid );

  wr = writer(fhvdya,VDYAsim);
  wr = writer(fhvdla,VDLAsim);

  loopct = loopct + 1;
  locate 10,1;
  "Loop           " loopct;
  "Invalid Draws  " sumc(VDinvalid);
  eofloop = eof(fhpara); 
  
endo;

closeall fhvdya, fhvdla, fhpara;
end;

/*********************************************************
**      P R O C E D U R E S
*/

proc (3) = vecspec(lambda,CCt,Phi0,sig);

local CCtt, sigma, Phi0t, 
      mu, mu_orth, la, la_orth, labar, laorthbar,
      phistar, phistar0, i, rr, nstate, Pt,
      MM1, MM2, ZZ, TT, S1, spec, gam, maxeigphi;

/* Expand the VAR model by Delta(L),
** assume that the constants are removed
** first column contains VEC coefficients
** fourth column contains coefficients on lagged L
*/

CCtt      = CCt | CCt[3,.]  ;
CCtt[4,4] = CCtt[4,4]-1     ;
sigma     = eye(3).*sig     ;

Phi0t     = Phi0 | Phi0[3,.];

/* construct mu_orth and la_orth
*/
mu =       CCtt[.,1];
mu_orth =  ( -mu[2] | mu[1] | 0 | 0 ) ~
           ( 0 | -mu[3] | mu[2] | 0 ) ~
           ( 0 | 0 | -mu[4] | mu[3] ) ;

la =       ( 1 | -lambda| 0 | 0 );
la_orth =  ( lambda | 1 | 0 | 0 ) ~
           ( 0      | 0 | 1 | 0 ) ~
           ( 0      | 0 | 0 | 1 );

labar     = la*inv(la'*la);
laorthbar = la_orth*inv(la_orth'*la_orth);


/* Set up companion form matrix
*/

phistar0 = labar ~ laorthbar;

i = 1;
phistar = inv(phistar0)*( (mu + labar + ( CCtt[.,2:4]~zeros(4,1) )* labar) ~ (CCtt[.,2:4]~zeros(4,1))*laorthbar);
if p > 1;
   do until i == p;
      phistar = phistar ~ inv(phistar0)* ( ( (CCtt[.,(1+nvar*i+1):(1+nvar*(i+1))]~zeros(4,1) )  
                                           - (CCtt[.,(1+nvar*(i-1)+1):(1+nvar*i)]~zeros(4,1) ) )*labar
                                       ~ (CCtt[.,(1+nvar*i+1):(1+nvar*(i+1))]~zeros(4,1))*laorthbar );
      i = i+1;
   endo;
endif;
phistar = phistar ~ inv(phistar0)* ( ( -CCtt[.,1+nvar*(i-1)+1:1+nvar*i] ~ zeros(4,1) )*labar ~ zeros(4,3) );

phi0t = inv(phistar0)*phi0t;

phistar = phistar | (eye(cols(phistar)-4) ~ zeros(cols(phistar)-4,4) );
rr      = phi0t   | zeros(cols(phistar)-4,3) ;
nstate  = cols(phistar);


Pt = reshape(inv(eye(nstate^2) - (Phistar.*.Phistar))*vec(RR*SIGMA*RR'),nstate,nstate);
maxeigphi = maxc(abs(eig(PHIstar)));

 
/* Define Selection Matrices
*/
MM1 = ( 1 ~ 0 ~ 0 ~ 1)|
      ( 0 ~ 0 ~ 1 ~ 0);

MM2 = zeros(4,nstate);
MM2[.,1:4] = labar ~ laorthbar;
MM2[.,5]   = -labar;

ZZ = MM1*MM2;
TT = phistar;

spec = zeros(2,nomega);

i = 1;
do until i > nomega;
   
   S1 = complex( eye(nstate) - TT*cos(omegagr[i]),   TT*sin(omegagr[i]) );  
   spec[.,i] = real( diag( ZZ* (inv(S1)*(RR*sigma*RR')*inv(S1)'/(2*pi) )* ZZ' ) );

   i = i+1;

endo;

Gam = ZZ*Pt*ZZ';

retp(spec,gam,maxeigphi);
endp;
