/***************************************************************/
/*                                                             */
/*     VEC analysis: sampling from the prior    ,              */
/*                                                             */
/*                       Frank Schorfheide                     */
/*                                                             */
/***************************************************************/

/* Filename:  vecprior.g
** created:   09/20/01
*/


closeall;
library pgraph;

#include c:\projects\active\homep\Gauss\pathspec.g
#include c:\projects\active\homep\Gauss\vecspec.g

/* Configuration
*/
lprior = "1";
lmodel = "m0";
mhrun  = "0";


/* Gibbs Sampler Setup
**
*/
nsim     = 110;
nblocks  = 1000;
la_scale = 0.5;

/* Import data on productivity growth, homeinvestment growth, and hours
**        observations from 1965:I to 1997:IV
*/
#include c:\projects\active\homep\Gauss\loaddata.g

nobs     = T0;       /* number of observations for training sample */
nvar     = cols(series);
npara    = nvar*(1+ttrend+1+p*nvar)+ 0.5*nvar*(nvar+1) + 1; 
YY       = series[1+p:nobs,.];
XXvec    = seriesvec[1+p:nobs,.];

if ttrend == 0;
   k     = 1 + nvar*p;            /* number of coefficients          */
   XX    = ones(nobs-p, k);
   if p > 0;
   i = 1;
   do until i > p;
     XX[.,2+(i-1)*nvar:1+i*nvar] = series[p-i+1:nobs-i,.];
     i = i+1;
   endo;
   endif;
else;
   k     = 2 + nvar*p;            /* number of coefficients          */
   XX    = ones(nobs-p, k);
   XX[.,2] = seqa(1,1,nobs-p);
   if p > 0;
   i = 1;
   do until i > p;
     XX[.,3+(i-1)*nvar:2+i*nvar] = series[p-i+1:nobs-i,.];
     i = i+1;
   endo;
   endif;
endif;

nobs    = nobs - p;

/* Initialize Output files
*/
opath = parapath $+ "\\mhrun" $+ mhrun $+ "\\";
opara = opath  $+ lmodel $+ lprior $+ "par";

create fhpara = ^opara with PARAM, npara, 8;
cls;


/*************************************************************
**           Posterior Simulator
*/

/* Initialize the Gibbs Sampler
*/
la_sim  = 1 + la_scale*rndn(1,1);


XXs     = (XXvec*(1|-la_sim) )~XX;
CC_sim  = inv(XXs'*XXs)*XXs'*YY;
CCt_sim = CC_sim';
Sig_sim = (YY-XXs*CC_sim)'*(YY-XXs*CC_sim)/nobs;

iblock = 1;
do until iblock > nblocks; 

  parasim   = zeros(nsim,npara);

  j = 1;
  do until j > nsim;

     /* Draw from lambda conditional on CC,Sigma
     */
     /*
     YYtrans = YY - XX*CC_sim[2:rows(CC_sim),.] - XXvec[.,1]*CC_sim[1,.];
     XXtrans = -XXvec[.,2]*CC_sim[1,.];
     vhat_la = 1/sumc(diag(inv(Sig_sim)*(XXtrans'*XXtrans)));
     mhat_la = vhat_la*sumc(diag(inv(Sig_sim)*(XXtrans'*YYtrans)));
     vT_la   = 1/(1/vhat_la + 1/v0_la);
     mT_la   = vT_la*(mhat_la/vhat_la + m0_la/v0_la);
     la_sim  = mT_la + sqrt(vT_la)*rndn(1,1);
     */
     la_sim  = m0_la + sqrt(v0_la)*rndn(1,1);     

     /* Draw from Sigma, CC conditional on lambda
     */
     XXs     = (XXvec*(1|-la_sim) )~XX;
     CC_hat  = inv(XXs'*XXs)*XXs'*YY;    
     H       = (YY-XXs*CC_hat)'*(YY-XXs*CC_hat);
     Z       = chol(inv(H))'*rndn(nvar,nobs-k-1);
     Sig_sim = inv(Z*Z');

     VC     = Sig_sim.*.inv(XXs'*XXs);
     C_sim  = vec(CC_hat) + cholpsd(VC)*rndn(nvar*(k+1),1);
     CCt_sim= reshape(C_sim,nvar,k+1);
     CC_sim = CCt_sim';

     parasim[j,.] = la_sim ~ Sig_sim[1,1:3] ~ Sig_sim[2,2:3] ~ Sig_sim[3,3] ~ C_sim'; 
              
     /* Screen output
     */
     /*
     locate 1,1;
     "Block          " iblock "out of" nblocks;
     "Simulation step" j "out of" nsim;
     "Cointvector    " la_sim;
     */
     j = j+1;

  endo;
 
  /* Save the results
  */
  wr = writer(fhpara,parasim);

  /* Screen output
  */
  
  locate 1,1;
  "Block          " iblock "out of" nblocks;
  

  iblock = iblock+1;
endo;

closeall fhpara;

