/***************************************************************/
/*                                                             */
/*    This program computes the RMSE at the posterior mean     */
/*                                                             */
/***************************************************************/

/* Filename:  vecrmse.g
** created:   10/01/01
*/

new; 
closeall;
library pgraph;
cls;

#include c:\projects\active\homep\Gauss\pathspec.g
#include c:\projects\active\homep\Gauss\vecspec.g

/* Configuration
*/
lprior = "1";
lmodel = "m0";
mhrun  = "1";

/* Import data on productivity growth, homeinvestment growth, and hours
**        observations from 1955:I to 1997:IV
*/
#include c:\projects\active\homep\Gauss\loaddata.g

nobs     = rows(series);       /* number of observations          */
nvar     = cols(series);
npara    = nvar*(1+ttrend+1+p*nvar)+ 0.5*nvar*(nvar+1) + 1; 
YY       = series[1+p:nobs,.];
XXvec    = seriesvec[1+p:nobs,.];


if ttrend == 0;
   k     = 1 + nvar*p;            /* number of coefficients          */
   XX    = ones(nobs-p, k);
   if p > 0;
   i = 1;
   do until i > p;
     XX[.,2+(i-1)*nvar:1+i*nvar] = series[p-i+1:nobs-i,.];
     i = i+1;
   endo;
   endif;
else;
   k     = 2 + nvar*p;            /* number of coefficients          */
   XX    = ones(nobs-p, k);
   XX[.,2] = seqa(1,1,nobs-p);
   if p > 0;
   i = 1;
   do until i > p;
     XX[.,3+(i-1)*nvar:2+i*nvar] = series[p-i+1:nobs-i,.];
     i = i+1;
   endo;
   endif;
endif;

nobs    = nobs - p;
T0      = T0 - p;

/* Open files with VEC Parameter Draws (Posterior Mean)
*/
lpath       = parapath $+ "\\mhrun" $+ mhrun $+ "\\";
lpara       = lpath $+ "\\" $+ lmodel  $+ lprior $+ "par" $+ "s";
open fhpara = ^lpara for read;
para        = readr( fhpara, 1 );
closeall fhpara;

la_hat  = para[1,1];

sig_hat = zeros(3,3);
sig_hat[1,1:3] = para[1,2:4];
sig_hat[1:3,1] = para[1,2:4]';
sig_hat[2,2:3] = para[1,5:6];
sig_hat[2:3,2] = para[1,5:6]';
sig_hat[3,3]   = para[1,7];
     
C_hat   = para[1,8:npara]';
CCt_hat = reshape(C_hat,nvar,k+1);
CC_hat  = CCt_hat';

XXs     = (XXvec*(1|-la_hat) )~XX;
prederror = (YY-XXs*CC_hat);
prederror = prederror[T0+1:rows(prederror),.];

"VEC Model";
"=========";
"";
"Number of Observations used to compute RMSE";
rows(prederror);
"";
"RMSE" sqrt(meanc(prederror^2));
